# copyright: Huynh Duong Phuong Vi
# www.sketchupforyou.com
def S4U::get_file_setting(dir_s4u_name,s4u_name)
	begin
		file_name = s4u_name + ".dat"
		file_name_full = File.join dir_s4u_name,file_name
		if File.exist?(file_name_full)
			file= File.new(file_name_full,"r")
			str=file.readlines
			file.close
			return str.collect{|s| s.chomp}
		end
		return false
	rescue
		return false
	end
end
def S4U::set_file_setting(dir_s4u_name,s4u_name,list)
	begin
		file_name = s4u_name + ".dat"
		file_name_full = File.join dir_s4u_name,file_name
		file= File.new(file_name_full,"w")
		list.each{|s| file.write(s.to_s+"\n")}
		file.close
		return true
	rescue
		return false
	end
end
module S4U
	module S4u_makebox
		@setting=[]
		class Box
			def initialize(box_mode=0)
				@box_mode=box_mode
				@golden_section=(1+Math.sqrt(5))/2
				@scale_factor= Sketchup.version.to_i > 17 ? UI.scale_factor : 1
				@snap=20*@scale_factor
				@strings=S4U::S4u_makebox::Strings
				cursor_path=["mouse_corner","mouse_middle","mouse_center"]
				if IS_OSX
					cursor_path.each_index{|i| cursor_path[i]+=".pdf"}
				else
					cursor_path.each_index{|i| cursor_path[i]+=".svg"}
				end
				@cursor_id = cursor_path.collect{|i| UI.create_cursor(File.join(Dir_s4u_name,i), 0,0)}
			end
			def activate
				model=Sketchup.active_model
				@edit_transform=axes_edit_transform
				@ip1=Sketchup::InputPoint.new
				@ip2=Sketchup::InputPoint.new
				@ip3=Sketchup::InputPoint.new
				@ip4=Sketchup::InputPoint.new
				@ip=Sketchup::InputPoint.new
				@faces=[]
				@element=nil
				@box_mode_list=[@strings.GetString("Corner"),@strings.GetString("Middle"),@strings.GetString("Center")]
				@make_group_list=[@strings.GetString("None"),@strings.GetString("Group"),@strings.GetString("Component")]
				@setting= S4U::S4u_makebox::instance_variable_get :@setting
				if @setting  && @setting.length==4
					i=-1
					@name=@setting[i+=1]
					@make_group=@setting[i+=1]
					@box_mode=@setting[i+=1]
					@thickness=@setting[i+=1]
				else
					self.reset_setting(S4U::S4u_makebox::NAME)
				end
				self.reset(model.active_view)
			end

			def axes_edit_transform
				model=Sketchup.active_model
				return model.axes.transformation if Sketchup.version.to_i>=16
				model.start_operation "get edit transform"
				ents=model.active_entities
				begin
					g=ents.add_group(ents.add_group)
					t=Geom::Transformation.new(g.transformation.xaxis,g.transformation.yaxis,g.transformation.zaxis,g.transformation.origin)
					g.erase! if g && g.valid? && !g.deleted?
				rescue
					t=model.edit_transform
				end
				model.abort_operation
				return t
			end
			def getExtents
				bb = Sketchup.active_model.bounds
				bb.add @ip.position if @ip.valid?
				bb.add(@p1) if @p1
				bb.add(@p2) if @p2
				bb.add(@p3) if @p3
				bb.add(@p4) if @p4
				return bb
			end
			def reset_setting(file=nil)
				model=Sketchup.active_model
				@name="Box"
				@make_group=1
				@box_mode=0
				@thickness=0.0
				begin
					if file && S4U::S4u_makebox::Dir_s4u_name && (s=S4U::get_file_setting(S4U::S4u_makebox::Dir_s4u_name,file))
						i=-1
						@name=s[i+=1].to_s
						@make_group=s[i+=1].to_i
						@box_mode=s[i+=1].to_i
						@thickness= parse_length(s[i+=1].to_s)
					else
						puts "no get setting"
					end
				rescue
					puts "error get setting"
				end
				materials=model.materials.collect{|m| m.name}
				@material=materials.index(model.materials.current.name) if materials.length>0 && model.materials.current
			end
			def onSetCursor
				UI.set_cursor(@cursor_id[@box_mode])
			end
			def deactivate(view)
				model=Sketchup.active_model
				if @explode && @elements && !@elements.deleted?
					model.start_operation @strings.GetString("Make Box"),true,false,true
						@elements.explode# if @explode && @elements && !@elements.deleted?
					model.commit_operation
				end
				i=-1
				@setting[i+=1]=@name
				@setting[i+=1]=@make_group
				@setting[i+=1]=@box_mode
				@setting[i+=1]=@thickness
				S4U::S4u_makebox::instance_variable_set(:@setting,@setting)
				(puts "error save setting") if not(S4U::S4u_makebox::Dir_s4u_name && S4U::set_file_setting(S4U::S4u_makebox::Dir_s4u_name,S4U::S4u_makebox::NAME,@setting))
			end
			def setstatus(view=nil)
				text = @strings.GetString("Box Mode") + ":" + ("Draw box from ")
				text+= @box_mode_list[@box_mode].upcase
				text+=" (" + @strings.GetString("Press TAB for change") + ") " + " | "
				text+= @strings.GetString("Make GC") + ": "
				text+= @make_group_list[@make_group].upcase
				text+=" (" + @strings.GetString("RightMouse for setting") + ")| "
					if @state==0
						text+=@strings.GetString("Select First point")
					elsif @state==1
						text+= @strings.GetString("Select Second point")
					elsif @state==2
						text+= @strings.GetString("Select Third point")
					end
				Sketchup.status_text=text
				view.invalidate if view
			end
			def getMenu(menu)
				menus=[]
				@box_mode_list.each_index{|i| menus<< menu.add_item(@box_mode_list[i]) {@box_mode=i; self.setstatus }}
				menus.each{|m| menu.set_validation_proc(m) { MF_UNCHECKED }}
				menu.set_validation_proc(menus[@box_mode]) { MF_CHECKED }
				menu.add_separator
				menus=[]
				@make_group_list.each_index{|i| menus<< menu.add_item(@make_group_list[i]) {@make_group=i; self.setstatus }}
				menus.each{|m| menu.set_validation_proc(m) { MF_UNCHECKED }}
				menu.set_validation_proc(menus[@make_group]) { MF_CHECKED }
				menu.add_separator
				menus=[]
				menus<< menu.add_item(@strings.GetString("Set parameter")) { self.setting ; self.setstatus }
				menus.each{|m| menu.set_validation_proc(m) { MF_UNCHECKED }}
			end
			def draw(view)
				model=view.model
				view.line_width=2
				view.drawing_color="red"
				if @ip && @ip.valid?
					if @ip.display?
						@ip.draw(view)
					elsif @face || @edge
						view.draw_points( @ip.position, 9*@scale_factor, 1, "red" )
					end
				end
				if @state==1
					if @ip1.valid? && @ip2.valid?
						@ip1.draw(view)
						view.drawing_color ="red"
						view.draw_line @ip1.position,@ip2.position
						if @ip_shift && @shift_press
							view.line_stipple = "."
							view.drawing_color ="blue"
							view.draw_line(@ip.position, @ip2.position)
							view.line_stipple = ""
							view.drawing_color ="red"
							p2_line=@ip.position.project_to_line([@ip1.position,@ip_shift])
							view.draw_points( p2_line, 9*@scale_factor, 2, "red" ) if p2_line
						end
					end
				elsif @state==2
					@ip1.draw(view)
					@ip2.draw(view)
					view.drawing_color ="red"
					view.draw_line @ip1.position,@ip2.position
					if @ip3 && @ip3.valid?
						@ip3.draw(view)
						if @point
							view.draw_points( @p3, 9*@scale_factor ,1, "red" )
							view.line_stipple = "."
							if @box_mode==0
								view.draw_line( @p1,@p3)
							elsif @box_mode==1
								view.draw_line( (@p1+(@p1-@p2)),(@p3))
							elsif @box_mode==2
								view.draw_line( (@p1+(@p1-@p3)),@p3)
							end
							view.line_stipple = ""
						end
						ps=get_rectangle([@ip1.position,@ip2.position,@p3])
						if ps && ps.length==4
							view.drawing_color="red"
							view.draw_polyline ps<<ps[0]
						end
					end
				elsif @state==3
					@ip1.draw(view)
					@ip2.draw(view)
					@ip3.draw(view)
					ps=get_rectangle([@ip1.position,@ip2.position,@ip3.position])
					if ps && ps.length==4
						view.drawing_color="red"
						view.draw_polyline(ps+[ps[0]])
					end
					if @ip4 && @ip4.valid?
						@ip4.draw(view)
						if @point
							view.draw_points @p4, 9*@scale_factor, 1, "red"
							view.line_stipple = "."
							if @box_mode==0 || @box_mode==1
								view.draw_line( @p2,@p4)
							elsif @box_mode==2
								view.draw_line( (@p2+(@p2-@p3)),@p4)
							end
							view.line_stipple = ""
						end
						ps1=get_points_box(ps)
						if ps1
							view.drawing_color="red"
							#view.draw_polyline (ps1[0]<<ps1[0][0])
							view.draw_polyline( (ps1[1]<<ps1[1][0]))
							view.draw_line( ps1[2])
						end
					end
				end
				self.setstatus
			end
			def get_rectangle(points1)
				points=points1.clone
				ps=[]
				#if points.length==3
					if @box_mode==0
						ps=points.clone
						vx=points[1]-points[0]
						p2_line=points[2].project_to_line([points[0],vx])
						vyt=points[2]-p2_line
						ps[2]=points[1] + vyt
						vy=ps[2]-points[1]
						ps[3]=points[0] + vy
						return ps
					elsif @box_mode==1
						vy=points[1]-points[0]
						p3_line=points[2].project_to_line([points[0],vy])
						vx=points[2]-p3_line
						ps[0]=points[0] + vy.reverse
						ps[1]=points[1]
						ps[2]=points[1] + vx
						ps[3]=ps[0] + vx
						return [ps[3],ps[0],ps[1],ps[2]]
					elsif @box_mode==2
						vx=points[1]-points[0]
						vy=points[2]-points[1]
						#p3_line=points[2].project_to_line([points[0],vy])
						#vxt=points[2]-p3_line
						#ps[3]=points[1] + vxt
						#vx=points[1]-ps[3]
						ps[0]=points[0] + (vx.reverse + vy.reverse)
						ps[1]=points[0] + (vx+vy.reverse)
						ps[2]=points[0] + vx + vy
						ps[3]=points[0] + (vx.reverse + vy)
						return [ps[3],ps[0],ps[1],ps[2]]
					end
				#end
				return nil
			end
			def get_points_box(points)
				vx=(points[1]-points[0])
				vy=(points[3]-points[0])
				vz=@p3.vector_to @p4
				points1=[];points2=[]
				points.each_index{|i| points1[i]=points[i]+vz }
				points.each_index{|i| points2.push points[i],points1[i]}
				return [points,points1,points2]
			end
			def set_pointonedge(x,y,view,ip,edge)
				p1=edge.start.position.clone
				p2=edge.end.position.clone
				return nil if edge.length < (view.pixels_to_model(5*@snap, p1))
				points= [Geom.linear_combination(2.0/3, p1,1.0/3, p2),Geom.linear_combination(1.0/3, p1,2.0/3, p2)]
				ph = view.pick_helper
				ph.init(x, y)
				picked = points.find{|p| ph.test_point((p.transform ip.transformation),x,y,@snap) }
				if picked
					ip1=Sketchup::InputPoint.new(picked.transform ip.transformation)
					ip.copy! ip1
					view.tooltip="       " + @strings.GetString("1/3")
					return ip
				end
				return nil
			end
			def set_special_point(x,y,view,ip)
				if ip.face
					cf=center_face(ip.face).transform ip.transformation
					ph = view.pick_helper
					ph.init(x, y)
					picked = ph.test_point(cf,x,y,@snap)
					if picked
						ip1=Sketchup::InputPoint.new(cf)
						@face=true
						view.tooltip="       " + @strings.GetString("Center face")
						ip.copy! ip1
					else
						@edge=ip.face.edges.find{|e| set_pointonedge(x,y,view,ip,e) }
					end
				elsif ip.edge
					@edge=set_pointonedge(x,y,view,ip,ip.edge)
				end
			end
			def find_special_point(x,y,view,p1,p2,ip,d1)
				if p1!=p2
					d1*=2 if @box_mode==1
					ph = view.pick_helper
					vec=(p2-p1)
					vec.length=d1
					p_square= p1 + vec
					screen_square=view.screen_coords p_square
					vec.length=d1*@golden_section
					p_golden1= p1 + vec
					screen_golden1=view.screen_coords p_golden1
					vec.length=d1/@golden_section
					p_golden2= p1 + vec
					screen_golden2=view.screen_coords p_golden2
					p=p2
					if ph.init(p_square.x, p_square.y) && ph.test_point(p2,screen_square.x, screen_square.y,(@snap-5))
						p2=p_square
						view.tooltip= "       " + @strings.GetString("Square")
					elsif ph.init(screen_golden1.x, screen_golden1.y) && ph.test_point(p2,screen_golden1.x, screen_golden1.y,(@snap-5))
						p2=p_golden1
						view.tooltip="       " + @strings.GetString("Golden section")
					elsif ph.init(screen_golden2.x, screen_golden2.y) && ph.test_point(p2,screen_golden2.x, screen_golden2.y,(@snap-5))
						p2=p_golden2
						view.tooltip="       " + @strings.GetString("Golden section")
					end
					if p!=p2
						ip.copy! Sketchup::InputPoint.new(p2)
						Sketchup::set_status_text( (p1.distance p2).to_s, SB_VCB_VALUE)
						return ip
					end
				end
			end
			def onMouseMove(flags, x, y, view)
					@face=@edge=@point=nil
					if @state==0
						@ip.pick view,x,y
						set_special_point(x,y,view,@ip)
						view.invalidate if @ip.valid?
					elsif @state==1
						@ip.pick view,x,y,@ip1
						set_special_point(x,y,view,@ip)
						if @shift_press && @ip_shift && @ip!=@ip1 && @ip2!=@ip1 && (@ip1.position!=@ip_shift)
							p2_line=@ip.position.project_to_line([@ip1.position,@ip_shift])
							@ip2.copy!(p2_line ? Sketchup::InputPoint.new(p2_line) : @ip)
						else
							@ip2.copy! @ip
						end
						@p2=@ip2.position
						view.invalidate if @ip.valid?
						length = @ip1.position.distance(@ip2.position)
						Sketchup::set_status_text( length.to_s, SB_VCB_VALUE)
					elsif @state==2	&& (@ip.position != @p1)
						@ip.pick view,x,y,@ip2
						set_special_point(x,y,view,@ip)
						if @ip.valid? && ((@ip.position.distance_to_line [@p1,@p2])>0)
							if @shift_press	&& @ip3.valid? && ((@ip3.position.distance_to_line [@p1,@p2])>0)
								p2_line=@ip3.position.project_to_line([@p1,@p2])
								vyt=@ip3.position-p2_line
								p3_line=@ip.position.project_to_line([@p2,vyt])
								vy=p3_line-@p2
								@p3=@p2 + vy
							else
								p2_line=@ip.position.project_to_line([@p1,@p2])
								vy=@ip.position-p2_line
								@p3=@p2 + vy
							end
							@ip3.copy! (Sketchup::InputPoint.new(@p3))
							view.invalidate if @ip3.valid?
							d=@p2.distance(@p1)
							ip=find_special_point(x,y,view,@p2,@p3,@ip,d)
							if ip
								@ip3.copy! ip
								@p3=@point=ip.position
								view.invalidate if @ip3.valid?
							end
							length = @ip2.position.distance(@ip3.position)
							Sketchup::set_status_text( length.to_s, SB_VCB_VALUE)
						end
					elsif @state==3
						@ip.pick view,x,y,@ip3
						set_special_point(x,y,view,@ip)
						plane=[@p1,(@p2-@p1).cross(@p3-@p2)]
						@p4=@p3
						if @ip.valid? && ((@ip.position.distance_to_plane(plane))>0)
							if @shift_press	&& @ip4.valid? && ((@ip4.position.distance_to_plane(plane))>0)
								p3_line=@ip4.position.project_to_plane(plane)
								vyt=@ip4.position-p3_line
								if @p3 && vyt.length>0
									p4_line=@ip.position.project_to_line([@p3,vyt])
									vy=p4_line-@p3
									@p4=@p3 + vy
								end
							else
								p3_line=@ip.position.project_to_plane(plane)
								vy=@ip.position-p3_line
								@p4=@p3 + vy
							end
							@ip4.copy! (Sketchup::InputPoint.new(@p4))
							view.invalidate if @ip4.valid?
							if @p3!=@p4
								d=@p2.distance(@p3)
								ip=find_special_point(x,y,view,@p3,@p4,@ip,d)
								if ip
									@ip4.copy! ip
									@p4=@point=ip.position
									view.invalidate if @ip4.valid?
								end
							end
							length = @p3.distance(@p4)
							Sketchup::set_status_text( length.to_s, SB_VCB_VALUE)
						end
					end
					rescue=>error
					p error
			end
			def onReturn(view)
				if @state>1
					process_box([@p1,@p2,@p3,@p4],1)
					self.reset(view)
				end
			end
			def onLButtonDoubleClick(flags, x, y, view)
				if @state==3
					self.reset(view)
					process_box([@p1,@p2,@p3,@p4],1)
				end
			end
			def onLButtonDown(flags, x, y, view)
				if @state==0
					if @ip.valid?
						@ip1.copy! @ip
						@p1=@ip1.position
						@state=1
						view.lock_inference if  view.inference_locked?
					end
				elsif @state==1 && @ip.valid? && (@ip1.position!=@ip.position)
					#@ip2.copy! @ip
					@state=2
					#@p2=@ip2.position
					view.lock_inference if  view.inference_locked?
				elsif @state==2 && @ip.valid? && (@p1!=@ip.position) && (@p2!=@ip.position)
					#@ip3.copy! @ip
					@state=3
					#@p3=@ip3.position
					view.lock_inference if  view.inference_locked?
				elsif @state==3
					#@ip4.copy! @ip
					#@p4=@ip4.position
					#@double=0
					self.reset(view)
					process_box([@p1,@p2,@p3,@p4])
				end
				self.setstatus(view)
			end
			def setting
				model=Sketchup.active_model
				prompts = [@strings.GetString("Name"),@strings.GetString("Draw mode"),@strings.GetString("Make GC")]
				title=@strings.GetString("Enter Parameter Box")
				list=["",@box_mode_list.join('|'),@make_group_list.join('|')]
				defaults = [@name.to_s,@box_mode_list[@box_mode],@make_group_list[@make_group]]
				input = UI.inputbox(prompts,defaults,list,title)
				if input
					i=-1
					@name=input[i+=1]
					@box_mode=@box_mode_list.index(input[i+=1])
					@make_group=@make_group_list.index(input[i+=1])
				end
			end
			def cal(input)
				lengthformat=Sketchup.active_model.options["UnitsOptions"]["LengthFormat"]
				if op=input[/[\+\-\/\*]/]
					begin
					if input[/[\+\-]/]
						index= input.rindex(/[\+\-]/)
					elsif input[/[\*\/]/]
						index= input.rindex(/[\*\/]/)
					end
					if index
						if index>0
							s=[input[0..(index-1)],input[(index+1)..-1]]
						else
							s=["0",input[(index+1)..-1]]
						end
						op=input[index]
						if (lengthformat==0 || lengthformat==2)
							if op=="+"
								value= cal(s[0]).to_s.to_f + cal(s[1]).to_s.to_f
							elsif op=="-"
								value= cal(s[0]).to_s.to_f - cal(s[1]).to_s.to_f
							elsif op=="*"
								value= cal(s[0]).to_s.to_f * cal(s[1]).to_s.to_f
							elsif op=="/"
								value= cal(s[0]).to_s.to_f / cal(s[1]).to_s.to_f
							end
						else
							if op=="+"
								value= cal(s[0]).to_f + cal(s[1]).to_f
							elsif op=="-"
								value= cal(s[0]).to_f - cal(s[1]).to_f
							elsif op=="*"
								value= cal(s[0]).to_f * cal(s[1]).to_f
							elsif op=="/"
								value= cal(s[0]).to_f / cal(s[1]).to_f
							end
						end
					end
					rescue
						value=parse_length(input)
					end
				else
					value = parse_length(input)
				end
				return parse_length(value)
			end
			def parse_length(input)

				(return 0) if !input
				input=input.to_s
				if !Sketchup.parse_length("1.1")
					input.gsub!(/\./, ',')
				else
					input.gsub!(/\,/, '.')
				end
				begin
					value=input.to_l
				rescue
					value=input.to_f
				end
				return value
			end
			def process_box(points,flag=0)
				model=Sketchup.active_model
				model.start_operation @strings.GetString("Make Box"),true,false,false
				if flag==2
					e=@elements
					if e.is_a?Sketchup::ComponentInstance
						e.definition.entities.clear!
					elsif e.is_a?Sketchup::Group
						e.erase! if e && e.valid? && !e.deleted?
					end
					@elements=nil
				elsif @explode && @elements && !@elements.deleted?
					@elements.explode
				end
				ps=get_rectangle(points)
				@elements=make_box(ps,flag)
				model.commit_operation
				self.reset(model.active_view)
				self.setstatus
				rescue=>error
				p error.backtrace.join('\n')
				model.abort_operation
			end
			def make_box(points,flag)
				return if points && points.uniq.length!=4
				model=Sketchup.active_model
				entities=model.active_entities
				vx=(points[1]-points[0])
				vy=(points[2]-points[1])
				vzt=(points[3]-points[2])
				vz=vx.cross vy
				g=entities.add_group
				g.transformation=Geom::Transformation.new(vx,vy,vz,points[0])
				ps=points.clone
				ps=ps.collect{|p| p.transform g.transformation.inverse}
				f=g.entities.add_face ps
				if f
					f.reverse!
					if flag==0 && @p4
						p4_p=@p4.project_to_line([@p3,vz])
						vz1=@p3.vector_to p4_p
						p3=@p3.transform g.transformation.inverse
						p4=p4_p.transform g.transformation.inverse
						@thickness=p3.distance p4
						if vz1.angle_between(vz) > Math::PI*0.5
							# t=g.transformation
							g.transformation=Geom::Transformation.new(vx,vy,vz1,points[0])
							# g.entities.transform_entities t.inverse*g.transformation.inverse,g.entities.to_a
							# @thickness=-@thickness
						end
					end
					f.pushpull(@thickness, false) if @thickness && @thickness!=0
				else
					return nil
				end
				if @make_group==0
					@explode=true
				else
					@explode=false
					if @make_group==2
						g=g.to_component
						g.definition.name=g.name=@name.to_s
					elsif @make_group==1
						g.definition.name=g.name=@name.to_s
					end
				end
				return g
			end
			def get_vertices(face)
				return nil if not( face && (face.is_a?Sketchup::Face))
				ver=[]
				vertices=[]
				vertices =face.outer_loop.vertices*3
				(0..face.outer_loop.vertices.length-1).each { |i|
					j=face.outer_loop.vertices.length+i
					p1=vertices[j-1].position.clone
					p2=vertices[j].position.clone
					p3=vertices[j+1].position.clone
					v1=p1.vector_to p2
					v2=p2.vector_to p3
					ver.push vertices[j] if not( v1.samedirection? v2)
				}
				return ver
			end
			def onKeyDown(key, repeat, flags, view)
				if key==CONSTRAIN_MODIFIER_KEY
					@shift_press=true
					if view.inference_locked?
						view.lock_inference
					elsif (@state==0 && @ip.valid?)
						view.lock_inference @ip
					elsif (@state==1 && @ip2.valid?)
						@ip_shift=@ip.position
						view.lock_inference @ip2,@ip1
					elsif @state==2 && @ip3.valid?
						@ip_shift= @ip3
						view.lock_inference @ip3,@ip2
					elsif @state==3 && @ip4.valid?
						@ip_shift= @ip4
						view.lock_inference @ip4,@ip3
					end
				elsif key==VK_UP
					view.lock_inference if view.inference_locked?
					if @ip.valid? && @state>0
						p= ((@state==1) ? @p1 : @p2)
						p1=p + @edit_transform.zaxis
						ip1 = Sketchup::InputPoint.new(p1)
						ip = Sketchup::InputPoint.new(p)
						view.lock_inference ip1,ip
					end
				elsif key==VK_LEFT
					view.lock_inference if view.inference_locked?
					if @ip.valid? && @state>0
						p=  ((@state==1) ? @p1 : @p2)
						p1=p + @edit_transform.yaxis
						ip1 = Sketchup::InputPoint.new(p1)
						ip = Sketchup::InputPoint.new(p)
						view.lock_inference ip1,ip
					end
				elsif key==VK_RIGHT
					view.lock_inference if view.inference_locked?
					if @ip.valid? && @state>0
						p= ((@state==1) ? @p1 : @p2)
						p1=p + @edit_transform.xaxis
						ip1 = Sketchup::InputPoint.new(p1)
						ip = Sketchup::InputPoint.new(p)
						view.lock_inference ip1,ip
					end
				end
				self.setstatus(view)
			end
			def onKeyUp(key, repeat, flags, view)
				if (key==CONSTRAIN_MODIFIER_KEY)
					@shift_press=false
					view.lock_inference if view.inference_locked?
				elsif key==VK_DOWN
					setting
				elsif (key==9) || ((key==48) && IS_OSX)
					@box_mode=(@box_mode==2) ? 0 : (@box_mode+1)
				#elsif key==VK_DOWN
					#@make_group=(@make_group==2) ? 0 : (@make_group+1)
				end
				self.setstatus(view)
				onSetCursor
			end
			def onUserText(text, view)
				if @state==1
					if @ip.position!=@p1
						vec=@ip.position - @p1
						length=self.cal(text)
						Sketchup::set_status_text( length.to_s, SB_VCB_VALUE)
						if length!=0 && vec.length>0
							vec.length=length
							@p2=@p1 + vec
							ip = Sketchup::InputPoint.new(@p2)
							@ip2.copy! ip
							@state=2
							view.lock_inference if  view.inference_locked?
						end
					end
				elsif @state==2
					if @ip.position!=@p2
						vx=@p2-@p1
						p2_line=@ip.position.project_to_line([@p1,vx])
						vy=@ip.position-p2_line
						length=self.cal(text)
						Sketchup::set_status_text( length.to_s, SB_VCB_VALUE)
						if  length!=0 && vy.length>0
							vy.length=length
							@p3=@p2 + vy
							ip = Sketchup::InputPoint.new(@p3)
							@ip3.copy! ip
							@state=3
							view.lock_inference if  view.inference_locked?
						end
					end
				elsif @state==3
					if @ip.position!=@p3
						vx=@p2-@p1
						vy=@p3-@p2
						p2_line=@ip.position.project_to_plane([@p1,vx.cross(vy)])
						vyt=@ip.position-p2_line
						length=self.cal(text)
						Sketchup::set_status_text( length.to_s, SB_VCB_VALUE)
						if  length!=0 && vyt.length>0
							vyt.length=length
							@p4=@p3 + vyt
							process_box([@p1,@p2,@p3,@p4])
						else
							@thickness=length
							process_box([@p1,@p2,@p3,@p4],1)
						end
						view.lock_inference if  view.inference_locked?
					end
				elsif @state==0 #&& @thickness
					length=self.cal(text)
					Sketchup::set_status_text( length.to_s, SB_VCB_VALUE)
					if @elements && (length!= @thickness)
						@thickness=length
						process_box([@p1,@p2,@p3,@p4],2)
					end
				end
				self.setstatus(view)
				rescue
				p "input error"
			end
			def center_face(face)
				vertices=get_vertices(face)
				return nil if !vertices
				numver=vertices.length
				cp=Geom::Point3d.new
				vertices.each{|v|
					cp.x=cp.x+v.position.x
					cp.y=cp.y+v.position.y
					cp.z=cp.z+v.position.z
				}
				cp.x=cp.x/ numver
				cp.y=cp.y/ numver
				cp.z=cp.z/ numver
				return cp
			end
			def onCancel(reason, view)
				self.reset(view)
			end
			def reset(view=nil)
				@state=0
				@faces.clear
				@face=@edge=nil
				@ip1.clear
				@ip2.clear
				@ip3.clear
				@ip4.clear
				@ip.clear
				view.lock_inference if view && view.inference_locked?
				self.setstatus(view)
			end
			def resume(view)
				self.setstatus(view)
			end
		end
		def self.main(box_mode=0)
			@box_mode=0 if !@box_mode
			@box_mode=box_mode if box_mode<4
			Sketchup.active_model.select_tool Box.new(@box_mode)
		end
	end
end